"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = function (obj, key, value) { return key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value: value }) : obj[key] = value; };
var __spreadValues = function (a, b) {
    for (var prop in b || (b = {}))
        if (__hasOwnProp.call(b, prop))
            __defNormalProp(a, prop, b[prop]);
    if (__getOwnPropSymbols)
        for (var _i = 0, _a = __getOwnPropSymbols(b); _i < _a.length; _i++) {
            var prop = _a[_i];
            if (__propIsEnum.call(b, prop))
                __defNormalProp(a, prop, b[prop]);
        }
    return a;
};
var __export = function (target, all) {
    for (var name in all)
        __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = function (to, from, except, desc) {
    if (from && typeof from === "object" || typeof from === "function") {
        var _loop_1 = function (key) {
            if (!__hasOwnProp.call(to, key) && key !== except)
                __defProp(to, key, { get: function () { return from[key]; }, enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
        };
        for (var _i = 0, _a = __getOwnPropNames(from); _i < _a.length; _i++) {
            var key = _a[_i];
            _loop_1(key);
        }
    }
    return to;
};
var __toCommonJS = function (mod) { return __copyProps(__defProp({}, "__esModule", { value: true }), mod); };
// src/index.ts
var src_exports = {};
__export(src_exports, {
    createDocumentAdapter: function () { return createDocumentAdapter; }
});
module.exports = __toCommonJS(src_exports);
// src/document_state.ts
var createInitialStateFactory = function () {
    var getInitialState = function (additionalState) { return __spreadValues({
        ids: [],
        entities: {}
    }, additionalState); };
    return { getInitialState: getInitialState };
};
// src/state_adapter.ts
var import_toolkit = require("@reduxjs/toolkit");
var createSingleArgumentStateOperator = function (mutator) {
    var operator = createStateOperator(function (arg, state) { return mutator(state); });
    var operation = function (state) { return operator(state, void 0); };
    return operation;
};
var createStateOperator = function (mutator) {
    var operation = function (state, arg) {
        var isPayloadActionArgument = function (arg2) { return arg2.payload !== void 0 && arg2.type !== void 0; };
        var runMutator = function (draft) {
            if (isPayloadActionArgument(arg)) {
                mutator(arg.payload, draft);
            }
            else {
                mutator(arg, draft);
            }
        };
        if ((0, import_toolkit.isDraft)(state)) {
            runMutator(state);
            return state;
        }
        return (0, import_toolkit.createNextState)(state, runMutator);
    };
    return operation;
};
var createStateAdapter = function (sort) {
    var addMany = function (newDocuments, state) {
        var models = newDocuments.filter(function (model) { return !(model._id.toHexString() in state.entities); });
        merge(models, state);
    };
    var addOne = function (document, state) { return addMany([document], state); };
    var removeAll = function (state) {
        Object.assign(state, {
            ids: [],
            entities: {}
        });
    };
    var removeMany = function (documentIds, state) {
        var didMutate = false;
        documentIds.forEach(function (documentId) {
            var key = documentId.toHexString();
            if (key in state.entities) {
                delete state.entities[key];
                didMutate = true;
            }
        });
        if (didMutate) {
            state.ids = state.ids.filter(function (id) { return id in state.entities; });
        }
    };
    var removeOne = function (documentId, state) { return removeMany([documentId], state); };
    var setMany = function (newDocuments, state) { return merge(newDocuments, state); };
    var setOne = function (document, state) { return setMany([document], state); };
    var setAll = function (newDocuments, state) {
        state.entities = {};
        state.ids = [];
        addMany(newDocuments, state);
    };
    var updateMany = function (updates, state) {
        var models = [];
        updates.forEach(function (update) { return takeUpdatedModel(models, update, state); });
        if (models.length !== 0) {
            merge(models, state);
        }
    };
    var updateOne = function (update, state) { return updateMany([update], state); };
    var upsertMany = function (newDocuments, state) { return merge(newDocuments, state); };
    var upsertOne = function (document, state) { return upsertMany([document], state); };
    var takeUpdatedModel = function (models, update, state) {
        var key = update.id.toHexString();
        if (!(key in state.entities)) {
            return false;
        }
        var original = state.entities[key];
        var updated = __spreadValues(__spreadValues({}, original), update.changes);
        var newKey = updated._id.toHexString();
        delete state.entities[key];
        models.push(updated);
        return newKey !== key;
    };
    var areIdsEqual = function (a, b) {
        if (a.length !== b.length) {
            return false;
        }
        for (var i = 0; i < a.length && i < b.length; i += 1) {
            if (a[i] !== b[i]) {
                return false;
            }
        }
        return true;
    };
    var updateIds = function (state) {
        var documents = Object.values(state.entities);
        if (sort) {
            documents.sort(sort);
        }
        var newIds = documents.map(function (document) { return document._id.toHexString(); });
        var ids = state.ids;
        if (!areIdsEqual(ids, newIds)) {
            state.ids = newIds;
        }
    };
    var merge = function (models, state) {
        if (models.length !== 0) {
            models.forEach(function (model) {
                state.entities[model._id.toHexString()] = model;
            });
            updateIds(state);
        }
    };
    return {
        addMany: createStateOperator(addMany),
        addOne: createStateOperator(addOne),
        removeAll: createSingleArgumentStateOperator(removeAll),
        removeMany: createStateOperator(removeMany),
        removeOne: createStateOperator(removeOne),
        setMany: createStateOperator(setMany),
        setOne: createStateOperator(setOne),
        setAll: createStateOperator(setAll),
        updateMany: createStateOperator(updateMany),
        updateOne: createStateOperator(updateOne),
        upsertMany: createStateOperator(upsertMany),
        upsertOne: createStateOperator(upsertOne)
    };
};
// src/state_selectors.ts
var import_toolkit2 = require("@reduxjs/toolkit");
var createSelectorsFactory = function () {
    var getSelectors = function () {
        var selectIds = function (state) { return state.ids; };
        var selectEntities = function (state) { return state.entities; };
        var selectAll = (0, import_toolkit2.createDraftSafeSelector)(selectIds, selectEntities, function (ids, entities) { return ids.map(function (id) { return entities[id]; }); });
        var selectId = function (state, id) { return id; };
        var selectById = function (entities, id) { return entities[id.toHexString()]; };
        var selectTotal = (0, import_toolkit2.createDraftSafeSelector)(selectIds, function (ids) { return ids.length; });
        return {
            selectIds: selectIds,
            selectEntities: selectEntities,
            selectAll: selectAll,
            selectTotal: selectTotal,
            selectById: (0, import_toolkit2.createDraftSafeSelector)(selectEntities, selectId, selectById)
        };
    };
    return { getSelectors: getSelectors };
};
// src/create_adapter.ts
var createDocumentAdapter = function (_a) {
    var _b = _a === void 0 ? {} : _a, sortComparer = _b.sortComparer;
    var stateFactory = createInitialStateFactory();
    var selectorsFactory = createSelectorsFactory();
    var stateAdapter = createStateAdapter(sortComparer);
    return __spreadValues(__spreadValues(__spreadValues({
        sortComparer: sortComparer
    }, stateFactory), selectorsFactory), stateAdapter);
};
//# sourceMappingURL=redux-document-adapter.cjs.development.js.map