var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
    for (var prop in b || (b = {}))
        if (__hasOwnProp.call(b, prop))
            __defNormalProp(a, prop, b[prop]);
    if (__getOwnPropSymbols)
        for (var prop of __getOwnPropSymbols(b)) {
            if (__propIsEnum.call(b, prop))
                __defNormalProp(a, prop, b[prop]);
        }
    return a;
};
// src/document_state.ts
var createInitialStateFactory = () => {
    const getInitialState = (additionalState) => __spreadValues({
        ids: [],
        entities: {}
    }, additionalState);
    return { getInitialState };
};
// src/state_adapter.ts
import { createNextState, isDraft } from "@reduxjs/toolkit";
var createSingleArgumentStateOperator = (mutator) => {
    const operator = createStateOperator((arg, state) => mutator(state));
    const operation = (state) => operator(state, void 0);
    return operation;
};
var createStateOperator = (mutator) => {
    const operation = (state, arg) => {
        const isPayloadActionArgument = (arg2) => arg2.payload !== void 0 && arg2.type !== void 0;
        const runMutator = (draft) => {
            if (isPayloadActionArgument(arg)) {
                mutator(arg.payload, draft);
            }
            else {
                mutator(arg, draft);
            }
        };
        if (isDraft(state)) {
            runMutator(state);
            return state;
        }
        return createNextState(state, runMutator);
    };
    return operation;
};
var createStateAdapter = (sort) => {
    const addMany = (newDocuments, state) => {
        const models = newDocuments.filter((model) => !(model._id.toHexString() in state.entities));
        merge(models, state);
    };
    const addOne = (document, state) => addMany([document], state);
    const removeAll = (state) => {
        Object.assign(state, {
            ids: [],
            entities: {}
        });
    };
    const removeMany = (documentIds, state) => {
        let didMutate = false;
        documentIds.forEach((documentId) => {
            const key = documentId.toHexString();
            if (key in state.entities) {
                delete state.entities[key];
                didMutate = true;
            }
        });
        if (didMutate) {
            state.ids = state.ids.filter((id) => id in state.entities);
        }
    };
    const removeOne = (documentId, state) => removeMany([documentId], state);
    const setMany = (newDocuments, state) => merge(newDocuments, state);
    const setOne = (document, state) => setMany([document], state);
    const setAll = (newDocuments, state) => {
        state.entities = {};
        state.ids = [];
        addMany(newDocuments, state);
    };
    const updateMany = (updates, state) => {
        const models = [];
        updates.forEach((update) => takeUpdatedModel(models, update, state));
        if (models.length !== 0) {
            merge(models, state);
        }
    };
    const updateOne = (update, state) => updateMany([update], state);
    const upsertMany = (newDocuments, state) => merge(newDocuments, state);
    const upsertOne = (document, state) => upsertMany([document], state);
    const takeUpdatedModel = (models, update, state) => {
        const key = update.id.toHexString();
        if (!(key in state.entities)) {
            return false;
        }
        const original = state.entities[key];
        const updated = __spreadValues(__spreadValues({}, original), update.changes);
        const newKey = updated._id.toHexString();
        delete state.entities[key];
        models.push(updated);
        return newKey !== key;
    };
    const areIdsEqual = (a, b) => {
        if (a.length !== b.length) {
            return false;
        }
        for (let i = 0; i < a.length && i < b.length; i += 1) {
            if (a[i] !== b[i]) {
                return false;
            }
        }
        return true;
    };
    const updateIds = (state) => {
        const documents = Object.values(state.entities);
        if (sort) {
            documents.sort(sort);
        }
        const newIds = documents.map((document) => document._id.toHexString());
        const { ids } = state;
        if (!areIdsEqual(ids, newIds)) {
            state.ids = newIds;
        }
    };
    const merge = (models, state) => {
        if (models.length !== 0) {
            models.forEach((model) => {
                state.entities[model._id.toHexString()] = model;
            });
            updateIds(state);
        }
    };
    return {
        addMany: createStateOperator(addMany),
        addOne: createStateOperator(addOne),
        removeAll: createSingleArgumentStateOperator(removeAll),
        removeMany: createStateOperator(removeMany),
        removeOne: createStateOperator(removeOne),
        setMany: createStateOperator(setMany),
        setOne: createStateOperator(setOne),
        setAll: createStateOperator(setAll),
        updateMany: createStateOperator(updateMany),
        updateOne: createStateOperator(updateOne),
        upsertMany: createStateOperator(upsertMany),
        upsertOne: createStateOperator(upsertOne)
    };
};
// src/state_selectors.ts
import { createDraftSafeSelector } from "@reduxjs/toolkit";
var createSelectorsFactory = () => {
    const getSelectors = () => {
        const selectIds = (state) => state.ids;
        const selectEntities = (state) => state.entities;
        const selectAll = createDraftSafeSelector(selectIds, selectEntities, (ids, entities) => ids.map((id) => entities[id]));
        const selectId = (state, id) => id;
        const selectById = (entities, id) => entities[id.toHexString()];
        const selectTotal = createDraftSafeSelector(selectIds, (ids) => ids.length);
        return {
            selectIds,
            selectEntities,
            selectAll,
            selectTotal,
            selectById: createDraftSafeSelector(selectEntities, selectId, selectById)
        };
    };
    return { getSelectors };
};
// src/create_adapter.ts
var createDocumentAdapter = ({ sortComparer } = {}) => {
    const stateFactory = createInitialStateFactory();
    const selectorsFactory = createSelectorsFactory();
    const stateAdapter = createStateAdapter(sortComparer);
    return __spreadValues(__spreadValues(__spreadValues({
        sortComparer
    }, stateFactory), selectorsFactory), stateAdapter);
};
export { createDocumentAdapter };
//# sourceMappingURL=redux-document-adapter.modern.js.map